@@ -24,7 +24,7 @@ cache at database 2, as opposed to database 1 with fetch.
 Set the environment variables QSV_FP_REDIS_CONNSTR, QSV_FP_REDIS_TTL_SECONDS and 
 QSV_FP_REDIS_TTL_REFRESH respectively to change default Redis settings.
 
-Supports brotli, gzip and deflate compression for improved throughtput & performance.
+Supports brotli, gzip and deflate compression for improved throughput & performance.
 Automatically upgrades its connection to HTTP/2 as well if the server supports it.
 (see https://www.cloudflare.com/learning/performance/http2-vs-http1.1/)
 
