@@ -1,18 +1,5 @@
 #![cfg(target_family = "unix")]
-use regex::bytes::{NoExpand, Regex};
-use std::ffi::OsStr;
-use std::io::BufReader;
-use std::os::unix::ffi::OsStrExt;
-use std::process::{Command, Stdio};
-
-use crate::config::{Config, Delimiter};
-use crate::select::SelectColumns;
-use crate::util;
-use crate::CliResult;
-use indicatif::{ProgressBar, ProgressDrawTarget};
-use serde::Deserialize;
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Execute a bash command once per line in given CSV file. Works only in
 Unix-like environments.
 
@@ -45,19 +32,36 @@ Common options:
                            headers.
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
-    -q, --quiet            Do not display progress bar.
-";
+    -p, --progressbar      Show progress bars. Not valid for stdin.
+"#;
+
+use std::{
+    ffi::OsStr,
+    io::BufReader,
+    os::unix::ffi::OsStrExt,
+    process::{Command, Stdio},
+};
+
+use indicatif::{ProgressBar, ProgressDrawTarget};
+use regex::bytes::{NoExpand, Regex};
+use serde::Deserialize;
+
+use crate::{
+    config::{Config, Delimiter},
+    select::SelectColumns,
+    util, CliResult,
+};
 
 #[derive(Deserialize)]
 struct Args {
-    arg_column: SelectColumns,
-    arg_command: String,
-    arg_input: Option<String>,
-    flag_unify: bool,
-    flag_new_column: Option<String>,
-    flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
-    flag_quiet: bool,
+    arg_column:       SelectColumns,
+    arg_command:      String,
+    arg_input:        Option<String>,
+    flag_unify:       bool,
+    flag_new_column:  Option<String>,
+    flag_no_headers:  bool,
+    flag_delimiter:   Option<Delimiter>,
+    flag_progressbar: bool,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -65,11 +69,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers)
+        .checkutf8(false)
         .select(args.arg_column);
 
     let mut rdr = rconfig.reader()?;
     let mut wtr = Config::new(&None).writer()?;
 
+    #[allow(clippy::trivial_regex)]
     let template_pattern = Regex::new(r"\{\}")?;
     let splitter_pattern = Regex::new(r#"(?:[\w-]+|"[^"]*"|'[^']*'|`[^`]*`)"#)?;
     let cleaner_pattern = Regex::new(r#"(?:^["'`]|["'`]$)"#)?;
@@ -82,16 +88,18 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut output_headers_written = false;
 
     // prep progress bar
-    let progress = ProgressBar::new(0);
-    if args.flag_quiet {
-        progress.set_draw_target(ProgressDrawTarget::hidden());
-    } else {
+    let show_progress =
+        (args.flag_progressbar || std::env::var("QSV_PROGRESSBAR").is_ok()) && !rconfig.is_stdin();
+
+    let progress = ProgressBar::with_draw_target(None, ProgressDrawTarget::stderr_with_hz(5));
+    if show_progress {
         util::prep_progress(&progress, util::count_rows(&rconfig)?);
+    } else {
+        progress.set_draw_target(ProgressDrawTarget::hidden());
     }
-    let not_quiet = !args.flag_quiet;
 
     while rdr.read_byte_record(&mut record)? {
-        if not_quiet {
+        if show_progress {
             progress.inc(1);
         }
         let current_value = &record[column_index];
@@ -108,26 +116,16 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             .map(|piece| {
                 let clean_piece = cleaner_pattern.replace_all(piece.as_bytes(), NoExpand(b""));
 
-                String::from_utf8(clean_piece.into_owned()).expect("encoding error")
+                String::from_utf8(clean_piece.into_owned()).unwrap_or_default()
             })
             .collect();
 
-        if !args.flag_unify {
-            let mut cmd = Command::new(prog)
-                .args(cmd_args)
-                .stdout(Stdio::inherit())
-                .stderr(Stdio::inherit())
-                .spawn()
-                .unwrap();
-
-            cmd.wait().unwrap();
-        } else {
+        if args.flag_unify {
             let mut cmd = Command::new(prog)
                 .args(cmd_args)
                 .stdout(Stdio::piped())
                 .stderr(Stdio::inherit())
-                .spawn()
-                .unwrap();
+                .spawn()?;
 
             {
                 let stdout = cmd.stdout.as_mut().unwrap();
@@ -164,10 +162,19 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 }
             }
 
+            cmd.wait().unwrap();
+        } else {
+            let mut cmd = Command::new(prog)
+                .args(cmd_args)
+                .stdout(Stdio::inherit())
+                .stderr(Stdio::inherit())
+                .spawn()
+                .unwrap();
+
             cmd.wait().unwrap();
         }
     }
-    if not_quiet {
+    if show_progress {
         util::finish_progress(&progress);
     }
     Ok(())
