@@ -28,6 +28,7 @@ Common options:
 
 use std::{
     fs, io,
+    io::prelude::*,
     path::{Path, PathBuf},
 };
 
@@ -54,5 +55,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut rdr = rconfig.reader_file()?;
     let mut wtr = io::BufWriter::new(fs::File::create(pidx)?);
     RandomAccessSimple::create(&mut rdr, &mut wtr)?;
+    wtr.flush()?;
+
     Ok(())
 }
