@@ -7,7 +7,7 @@ random indexing if the sample size is less than 10% of the total number of recor
 This allows for efficient sampling such that the entire CSV file is not parsed.
 
 When sample-size is between 0 and 1 exclusive, it is treated as a percentage
-of the CSV to sample (e.g. 0.20 is 20 percent). This requires an index.
+of the CSV to sample (e.g. 0.20 is 20 percent).
 
 This command is intended to provide a means to sample from a CSV data set that
 is too big to fit into memory (for example, for use with commands like 'qsv
@@ -23,7 +23,7 @@ Usage:
     qsv sample --help
 
 sample options:
-    --seed <number>        RNG seed.
+    --seed <number>        Random number generator seed.
 
 Common options:
     -h, --help             Display this message
@@ -60,10 +60,16 @@ struct Args {
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
+
+    if args.arg_sample_size.is_sign_negative() {
+        return fail!("Sample size cannot be negative.");
+    }
+
     let rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
         .checkutf8(false)
         .no_headers(args.flag_no_headers);
+
     let mut sample_size = args.arg_sample_size;
 
     let mut wtr = Config::new(&args.flag_output).writer()?;
@@ -82,8 +88,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         }
     } else {
         debug!("no index");
+        #[allow(clippy::cast_precision_loss)]
         if sample_size < 1.0 {
-            return fail!("Percentage sampling requires an index.");
+            let Ok(row_count) = util::count_rows(&rconfig) else {
+                return fail!("Cannot get rowcount. Percentage sampling requires a rowcount.");
+            };
+            sample_size *= row_count as f64;
         }
         let mut rdr = rconfig.reader()?;
         rconfig.write_headers(&mut rdr, &mut wtr)?;
@@ -108,7 +118,7 @@ where
     let mut all_indices = (0..idx.count()).collect::<Vec<_>>();
     let mut rng = ::rand::thread_rng();
     // this non-cryptographic shuffle is sufficient for our use case
-    // as we're optimizing for performance
+    // as we're optimizing for performance. Add DevSkim lint ignore.
     SliceRandom::shuffle(&mut *all_indices, &mut rng); //DevSkim: ignore DS148264
 
     let mut sampled = Vec::with_capacity(sample_size as usize);
@@ -133,7 +143,7 @@ fn sample_reservoir<R: io::Read>(
         reservoir.push(row?);
     }
 
-    // Seeding rng
+    // Seeding RNG
     let mut rng: StdRng = match seed {
         None => StdRng::from_rng(rand::thread_rng()).unwrap(),
         // the non-cryptographic seed_from_u64 is sufficient for our use case
