@@ -90,6 +90,7 @@ fn main() -> QsvExitCode {
     cat         Concatenate by row or column
     count       Count records
     dedup       Remove redundant rows
+    diff        Create the difference between two CSVs
     enum        Add a new column enumerating CSV lines
     excel       Exports an Excel sheet to a CSV
     exclude     Excludes the records in one CSV from another
@@ -261,6 +262,7 @@ enum Command {
     Cat,
     Count,
     Dedup,
+    Diff,
     Enum,
     Excel,
     Exclude,
@@ -333,6 +335,7 @@ impl Command {
             Command::Cat => cmd::cat::run(argv),
             Command::Count => cmd::count::run(argv),
             Command::Dedup => cmd::dedup::run(argv),
+            Command::Diff => cmd::diff::run(argv),
             Command::Enum => cmd::enumerate::run(argv),
             Command::Excel => cmd::excel::run(argv),
             Command::Exclude => cmd::exclude::run(argv),
