@@ -9,13 +9,14 @@ use std::{
 use docopt::Docopt;
 #[cfg(any(feature = "full", feature = "lite"))]
 use indicatif::{HumanCount, ProgressBar, ProgressStyle};
+use once_cell::sync::OnceCell;
 use serde::de::DeserializeOwned;
 #[cfg(any(feature = "full", feature = "lite"))]
 use serde::de::{Deserialize, Deserializer, Error};
 
 use crate::{
     config::{Config, Delimiter},
-    CliResult,
+    CliError, CliResult,
 };
 
 #[macro_export]
@@ -26,6 +27,8 @@ macro_rules! regex_once_cell {
     }};
 }
 
+static ROW_COUNT: once_cell::sync::OnceCell<u64> = OnceCell::new();
+
 #[inline]
 pub fn num_cpus() -> usize {
     num_cpus::get()
@@ -85,8 +88,25 @@ pub fn version() -> String {
     enabled_features.push_str("foreach;");
     #[cfg(all(feature = "generate", not(feature = "lite")))]
     enabled_features.push_str("generate;");
+
     #[cfg(all(feature = "luau", not(feature = "lite")))]
-    enabled_features.push_str("luau;");
+    {
+        let luau = mlua::Lua::new();
+        match luau.load("return _VERSION").eval() {
+            Ok(version_info) => {
+                if let mlua::Value::String(string_val) = version_info {
+                    enabled_features.push_str(&format!(
+                        "{};",
+                        string_val.to_str().unwrap_or("Luau - unknown version")
+                    ));
+                } else {
+                    enabled_features.push_str("Luau - ?;");
+                }
+            }
+            Err(e) => enabled_features.push_str(&format!("Luau - cannot retrieve version: {e};")),
+        };
+    }
+
     #[cfg(all(feature = "python", not(feature = "lite")))]
     {
         enabled_features.push_str("python-");
@@ -157,19 +177,31 @@ pub fn show_env_vars() {
 }
 
 #[inline]
-pub fn count_rows(conf: &Config) -> Result<u64, io::Error> {
+pub fn count_rows(conf: &Config) -> Result<u64, CliError> {
     if let Some(idx) = conf.indexed().unwrap_or(None) {
         Ok(idx.count())
     } else {
         // index does not exist or is stale,
         // count records by iterating through records
-        let mut rdr = conf.reader()?;
-        let mut count = 0u64;
-        let mut record = csv::ByteRecord::new();
-        while rdr.read_byte_record(&mut record)? {
-            count += 1;
+        // however, do this only once per invocation and cache the result
+        let rc = ROW_COUNT.get_or_init(|| {
+            if let Ok(mut rdr) = conf.reader() {
+                let mut count = 0u64;
+                let mut record = csv::ByteRecord::new();
+                while rdr.read_byte_record(&mut record).unwrap_or_default() {
+                    count += 1;
+                }
+                count
+            } else {
+                u64::MAX
+            }
+        });
+
+        if *rc < u64::MAX {
+            Ok(*rc)
+        } else {
+            Err(CliError::Other("Unable to get row count".to_string()))
         }
-        Ok(count)
     }
 }
 
